package com.landenlabs.devstuff;

/**
 * Copyright (c) 2016 Dennis Lang (LanDen Labs) landenlabs@gmail.com
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
 * associated documentation files (the "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * @author Dennis Lang  (3/21/2015)
 * @see http://LanDenLabs.com/
 *
 */


import android.app.ActivityManager;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.graphics.LinearGradient;
import android.graphics.Paint;
import android.graphics.RectF;
import android.graphics.Shader;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.BatteryManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.SystemClock;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.SubMenu;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Spinner;
import android.widget.TextView;

import com.androidplot.Plot;
import com.androidplot.Series;
import com.androidplot.util.Redrawer;
import com.androidplot.xy.BoundaryMode;
import com.androidplot.xy.LineAndPointFormatter;
import com.androidplot.xy.SimpleXYSeries;
import com.androidplot.xy.XYGraphWidget;
import com.androidplot.xy.XYPlot;
import com.androidplot.xy.XYStepMode;
import com.landenlabs.devstuff.util.SoundMeter;
import com.landenlabs.devstuff.util.Ui;
import com.landenlabs.devstuff.util.Utils;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Display "Sensor" information.
 * 
 * @author Dennis Lang
 *
 */
public class SensorFragment extends DevFragment implements SensorEventListener, View.OnLayoutChangeListener, AdapterView.OnItemSelectedListener {

    View m_rootView;
    TextView m_valueTv1;

    // ---- Plot
    private static final int HISTORY_SIZE = 300;            // number of points to plot in history
    private XYPlot m_plot = null;
    private SimpleXYSeries[] m_series;
    private Redrawer m_redrawer;

    // ----- Sensor selector
    private SensorManager m_sensorMgr;
    List<Sensor> m_sensorList = new ArrayList<Sensor>();
    private Map<String, SensorEvent> m_sensorEvents = new HashMap<String, SensorEvent>();
    Map<String, String> m_plotValues = new HashMap<String, String>();

    Spinner m_consoleSpinner;
    SoundMeter m_soundMeter;

    private static final String WIFI_STR = "WiFi Level";
    private static final String BATTERY_STR = "Battery";
    private static final String ORIENTATION_STR = "Orientation";
    private static final String LIGHT_STR = "Light";
    private static final String AUDIO_STR = "Audio Db";
    private static final String PROCESSES_STR = "#Processes";
    private static final String MEMORY_STR = "Free Memory";

    private static final String[] SENSOR_NAMES = { WIFI_STR, BATTERY_STR, AUDIO_STR, ORIENTATION_STR, LIGHT_STR, PROCESSES_STR, MEMORY_STR  };
    String m_sensorName = WIFI_STR;

    final int ORIENTATION_AZ_SERIES = 0;
    final int ORIENTATION_PITCH_SERIES = 1;
    final int ORIENTATION_ROLL_SERIES = 2;
    final int SINGLE_SERIES = 0;

    static SimpleXYSeries m_seriesWifi;
    static SimpleXYSeries m_seriesBattery;
    static SimpleXYSeries m_seriesAudio;
    static SimpleXYSeries m_seriesAudioAvg;
    static SimpleXYSeries m_seriesAudioMax;
    static SimpleXYSeries m_seriesOrientation_az;
    static SimpleXYSeries m_seriesOrientation_pitch;
    static SimpleXYSeries m_seriesOrientation_roll;
    static SimpleXYSeries m_seriesLight;
    static SimpleXYSeries m_seriesProcCnt;
    static SimpleXYSeries m_seriesFreeMem;

    long m_maxAudio = 1000;
    final int MAX_LIGHT = 250;

    long m_sleepMsec = 100;
    int m_menuSelected = R.id.sensor_menu_100_msec;
    String m_domainLabel = "1/100 sec";
    static SparseArray<Integer> m_menuIdToMsec;
    static {
        m_menuIdToMsec = new SparseArray<Integer>();
        m_menuIdToMsec.put(R.id.sensor_menu_100_msec, 100);
        m_menuIdToMsec.put(R.id.sensor_menu_1_second, 1000*1);
        m_menuIdToMsec.put(R.id.sensor_menu_5_seconds, 1000*5);
        m_menuIdToMsec.put(R.id.sensor_menu_1_minute, 1000*60*1);
        m_menuIdToMsec.put(R.id.sensor_menu_10_minutes,1000*60*10);
    }

    static SparseArray<String> m_menuIdToLbl;
    static {
        m_menuIdToLbl = new SparseArray<String>();
        m_menuIdToLbl.put(R.id.sensor_menu_100_msec, "1/100 Sec");
        m_menuIdToLbl.put(R.id.sensor_menu_1_second, "Seconds");
        m_menuIdToLbl.put(R.id.sensor_menu_5_seconds, "5 Sec");
        m_menuIdToLbl.put(R.id.sensor_menu_1_minute, "Minutes");
        m_menuIdToLbl.put(R.id.sensor_menu_10_minutes, "10 Min");
    }

    // ---- Timer
    private Handler m_handler = new Handler();
    private Runnable m_updateElapsedTimeTask = new Runnable() {
        public void run() {
            updatePlot();
            m_handler.postDelayed(this, m_sleepMsec);   // Re-execute after 1000 ms.
        }
    };

    // -----
    public static String s_name = "Sensor";

	public SensorFragment() {
	}
	
	public static DevFragment create() {
		return new SensorFragment();
	}

	// ============================================================================================
	// DevFragment methods
	
	@Override
	public String getName() {
		return s_name;
	}
	
	@Override
	public List<Bitmap> getBitmaps(int maxHeight) {
        List<Bitmap> bitmapList = new ArrayList<Bitmap>();
        bitmapList.add(Utils.grabScreen(this.getActivity()));
        return bitmapList;
	}
	
	@SuppressWarnings("deprecation")
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
       //  getActivity().setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

        m_sensorMgr = (SensorManager) getActivity().getSystemService(Context.SENSOR_SERVICE);
		m_rootView = inflater.inflate(R.layout.sensor_tab, container, false);
        m_valueTv1 = Ui.viewById(m_rootView, R.id.sensor_value);

        // Make all series - update them all the time
        if (false && m_seriesOrientation_az == null) {
            m_seriesOrientation_az = new SimpleXYSeries("Az.");
            m_seriesOrientation_az.useImplicitXVals();
            m_seriesOrientation_pitch = new SimpleXYSeries("Pitch");
            m_seriesOrientation_pitch.useImplicitXVals();
            m_seriesOrientation_roll = new SimpleXYSeries("Roll");
            m_seriesOrientation_roll.useImplicitXVals();
            m_seriesLight = new SimpleXYSeries("Lux");
            m_seriesLight.useImplicitXVals();
            m_seriesAudio = new SimpleXYSeries("Sound");
            m_seriesAudio.useImplicitXVals();
            m_seriesAudioAvg = new SimpleXYSeries("Avg");
            m_seriesAudioAvg.useImplicitXVals();
            m_seriesAudioMax = new SimpleXYSeries("Max");
            m_seriesAudioMax.useImplicitXVals();
            m_seriesProcCnt = new SimpleXYSeries("Proc");
            m_seriesProcCnt.useImplicitXVals();
            m_seriesFreeMem = new SimpleXYSeries("Mem");
            m_seriesFreeMem.useImplicitXVals();
            m_seriesBattery = new SimpleXYSeries("Battery");
            m_seriesBattery.useImplicitXVals();
            m_seriesWifi = new SimpleXYSeries("WiFi");
            m_seriesWifi.useImplicitXVals();
        }

        m_sensorList.add(m_sensorMgr.getDefaultSensor(Sensor.TYPE_ORIENTATION));
        m_sensorList.add(m_sensorMgr.getDefaultSensor(Sensor.TYPE_LIGHT));

        m_soundMeter = null; //  = new SoundMeter();

        // createGraph(m_rootView, savedInstanceState);
        XYPlot plot = Ui.viewById(m_rootView, R.id.plot);
        m_redrawer = new Redrawer(
                Arrays.asList(new Plot[]{plot /* ,m_plot2 */}), 100, false);

        m_consoleSpinner = Ui.viewById(m_rootView, R.id.sensor_spinner);
        List<String> sensorNames = new ArrayList<String>(Arrays.asList(SENSOR_NAMES));
        if (Build.VERSION.SDK_INT < 16) {
            // Total Memory not available < 16
            sensorNames.remove(MEMORY_STR);
        }

        //  ToDo - add multi-checkboxes instead of single.
        ArrayAdapter<String> spinnerArrayAdapter =
                new ArrayAdapter<String>(m_rootView.getContext(), android.R.layout.simple_spinner_item, sensorNames);
        spinnerArrayAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        m_consoleSpinner.setAdapter(spinnerArrayAdapter);
        m_consoleSpinner.addOnLayoutChangeListener(this);

        return m_rootView;
	}

    @Override
    public void onResume() {
        super.onResume();
        m_redrawer.start();
        m_handler.removeCallbacks(m_updateElapsedTimeTask);
        m_handler.postDelayed(m_updateElapsedTimeTask, 0);
        m_plot = null;
    }

    @Override
    public void onPause() {
        m_redrawer.pause();
        m_handler.removeCallbacks(m_updateElapsedTimeTask);
        super.onPause();
    }

    @Override
    public void onDestroy() {
        // m_redrawer.finish();
        super.onDestroy();
    }

    @Override
	public void onStart() {
		super.onStart();
        if (m_soundMeter != null)
            m_soundMeter.start();
        for (Sensor sensor : m_sensorList) {
            if (sensor != null) {
                m_sensorMgr.registerListener(this, sensor, SensorManager.SENSOR_DELAY_GAME);
            }
        }
    }

	@Override
	public void onStop() {
        detachSeries();
        if (m_soundMeter != null)
            m_soundMeter.stop();

        if (m_sensorMgr != null)
            m_sensorMgr.unregisterListener(this);
		super.onStop();
	}

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        switch (item.getItemId()) {
            case R.id.sensor_menu_100_msec:
            case R.id.sensor_menu_1_second:
            case R.id.sensor_menu_5_seconds:
            case R.id.sensor_menu_1_minute:
            case R.id.sensor_menu_10_minutes:
                m_menuSelected = id;
                m_sleepMsec = m_menuIdToMsec.get(id).intValue();
                m_domainLabel = m_menuIdToLbl.get(id);
                m_plot.setDomainLabel(m_domainLabel);
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onPrepareOptionsMenu(Menu menu) {
        super.onPrepareOptionsMenu(menu);

        menu.findItem(m_menuSelected).setChecked(true);
    }

    SubMenu m_menu;
    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        super.onCreateOptionsMenu(menu, inflater);
        m_menu = menu.addSubMenu("Sensor");
        inflater.inflate(R.menu.sensor_menu, m_menu);
    }

    // ============================================================================================
    // implement OnLayoutChangeListener
    @Override
    public void onLayoutChange(View v, int left, int top, int right,
                               int bottom, int oldLeft, int oldTop, int oldRight, int oldBottom) {
        if (v == m_consoleSpinner)
            m_consoleSpinner.setOnItemSelectedListener(this);
    }

    // ============================================================================================
    // implement onItemSelectedListener
    @Override
    public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) {
        String itemStr = parent.getItemAtPosition(pos).toString();
        if (parent == m_consoleSpinner) {
            if (!itemStr.equals(m_sensorName)) {
                m_sensorName = itemStr;
                // createGraph(m_rootView, null);
                m_plot = null;
            }
        }
    }

    @Override
    public void onNothingSelected(AdapterView<?> arg0) {
    }

    // ============================================================================================
    // Graph Stuff

    public void detachSeries() {
        if (m_plot != null) {
            for (Series series : m_plot.getSeriesSet()) {
                SimpleXYSeries xySeries = (SimpleXYSeries) series;
                m_plot.removeSeries(xySeries);
            }
        }
    }

    public void createGraph(View rootView, Bundle savedInstanceState) {

        final int lineOnlyWidth = 6;
        final int lineFillWidth = 6;

        // setup the APR History plot:
        m_plot = Ui.viewById(rootView, R.id.plot);
        detachSeries();

        m_sensorName = m_consoleSpinner.getSelectedItem().toString();
        m_plot.setTitle(m_sensorName);

        // Set the plot background (do once)
        XYGraphWidget wid = m_plot.getGraphWidget();
        Paint bg = wid.getBackgroundPaint();
        RectF rectF = wid.getGridRect();
        float width = (rectF != null) ? rectF.right : 250;
        float height = (rectF != null) ? rectF.height() : 250;
        Shader colorShader = new LinearGradient(0, 0, width, 0, Color.BLACK, Color.rgb(0, 64, 0), Shader.TileMode.MIRROR);
        m_plot.getGraphWidget().getGridBackgroundPaint().setShader(colorShader);
        m_plot.setRangeStep(XYStepMode.SUBDIVIDE, 10 + 1);
        m_plot.setTicksPerRangeLabel(2);
        m_plot.setDomainLabel(m_domainLabel);


        final boolean LINE_MODE = false;
        final boolean FILL_MODE = true;
        if (m_sensorName.equals(ORIENTATION_STR)) {
            m_plot.setRangeBoundaries(-180, 359, BoundaryMode.FIXED);
            m_plot.setRangeLabel("Angle (Degs)");

            if (m_seriesOrientation_az == null) {
                m_seriesOrientation_az = new SimpleXYSeries("Az.");
                m_seriesOrientation_az.useImplicitXVals();
                m_seriesOrientation_pitch = new SimpleXYSeries("Pitch");
                m_seriesOrientation_pitch.useImplicitXVals();
                m_seriesOrientation_roll = new SimpleXYSeries("Roll");
                m_seriesOrientation_roll.useImplicitXVals();
            }

            m_plot.addSeries(m_seriesOrientation_az, makeFormatter(lineOnlyWidth, Color.rgb(40, 40, 255), Color.BLUE, 0,  height/3, LINE_MODE));
            m_plot.addSeries(m_seriesOrientation_pitch, makeFormatter(lineOnlyWidth, Color.rgb(40, 255, 40), Color.GREEN, 0, height/3, LINE_MODE));
            m_plot.addSeries(m_seriesOrientation_roll, makeFormatter(lineOnlyWidth, Color.rgb(255, 40, 40), Color.RED, 0, height/3, LINE_MODE));
        } else if (m_sensorName.equals(LIGHT_STR)) {

            if (m_seriesLight == null) {
                m_seriesLight = new SimpleXYSeries("Lux");
                m_seriesLight.useImplicitXVals();
            }

            m_plot.setRangeBoundaries(0, MAX_LIGHT, BoundaryMode.FIXED);
            m_plot.setRangeLabel("LUX");

            m_plot.addSeries(m_seriesLight, makeFormatter(lineFillWidth, Color.rgb(128, 0, 0), Color.WHITE, 0, height, FILL_MODE));
        } else if (m_sensorName.equals(AUDIO_STR)) {

            if (m_seriesAudio == null) {
                m_seriesAudio = new SimpleXYSeries("Sound");
                m_seriesAudio.useImplicitXVals();
                m_seriesAudioAvg = new SimpleXYSeries("Avg");
                m_seriesAudioAvg.useImplicitXVals();
                m_seriesAudioMax = new SimpleXYSeries("Max");
                m_seriesAudioMax.useImplicitXVals();
            }

            if (m_soundMeter == null) {
                m_soundMeter = new SoundMeter();
                m_soundMeter.start();
            }

            m_plot.setRangeBoundaries(0, m_maxAudio, BoundaryMode.FIXED);
            m_plot.setRangeLabel("Sound");

            m_plot.addSeries(m_seriesAudio, makeFormatter(2, Color.WHITE, Color.RED, 0, height, LINE_MODE));
            m_plot.addSeries(m_seriesAudioAvg, makeFormatter(1, Color.BLUE, Color.BLUE, 0, height, FILL_MODE));
            // m_plot.addSeries(m_seriesAudioMax, makeFormatter(4, Color.RED, Color.RED, 0, height, LINE_MODE));
        } else if (m_sensorName.equals(PROCESSES_STR)) {
            if (m_seriesProcCnt == null) {
                m_seriesProcCnt = new SimpleXYSeries("Proc");
                m_seriesProcCnt.useImplicitXVals();
            }

            m_plot.setRangeBoundaries(0, 100, BoundaryMode.GROW);
            m_plot.setRangeLabel("#Processes%");

            m_plot.addSeries(m_seriesProcCnt, makeFormatter(lineFillWidth, Color.YELLOW, Color.RED, 0, height, FILL_MODE));
        } else if (m_sensorName.equals(MEMORY_STR)) {

            if (m_seriesFreeMem == null) {
                m_seriesFreeMem = new SimpleXYSeries("Mem");
                m_seriesFreeMem.useImplicitXVals();
            }

            m_plot.setRangeBoundaries(0, 100, BoundaryMode.FIXED);
            m_plot.setRangeLabel("Memory Free %");

            m_plot.addSeries(m_seriesFreeMem, makeFormatter(lineFillWidth, Color.RED, Color.YELLOW, 0, height, FILL_MODE));
        } else if (m_sensorName.equals(BATTERY_STR)) {
            if (m_seriesBattery == null) {
                m_seriesBattery = new SimpleXYSeries("Battery");
                m_seriesBattery.useImplicitXVals();
            }

            m_plot.setRangeBoundaries(0, 110, BoundaryMode.FIXED);
            m_plot.setRangeStep(XYStepMode.INCREMENT_BY_VAL, 10);
            m_plot.setTicksPerRangeLabel(2);
            m_plot.setRangeLabel("Battery%");

            m_plot.addSeries(m_seriesBattery, makeFormatter(lineFillWidth, Color.rgb(255,128,0), Color.GREEN, 0, height, FILL_MODE));
        } else if (m_sensorName.equals(WIFI_STR)) {
            if (m_seriesWifi == null) {
                m_seriesWifi = new SimpleXYSeries("WiFi");
                m_seriesWifi.useImplicitXVals();
            }

            m_plot.setRangeBoundaries(0, 110, BoundaryMode.FIXED);
            m_plot.setRangeStep(XYStepMode.SUBDIVIDE, 11 + 1);
            m_plot.setTicksPerRangeLabel(2);

            m_plot.setRangeLabel("WiFi Signal%");
            m_plot.addSeries(m_seriesWifi, makeFormatter(lineFillWidth, Color.BLUE, Color.WHITE, 0, height, FILL_MODE));
        }

        m_plot.setDomainBoundaries(0, HISTORY_SIZE, BoundaryMode.FIXED);
        m_plot.setDomainStepMode(XYStepMode.INCREMENT_BY_VAL);
        m_plot.setDomainStepValue(HISTORY_SIZE/10);

        m_plot.getDomainLabelWidget().pack();
        m_plot.getRangeLabelWidget().pack();

        m_plot.setRangeValueFormat(new DecimalFormat("#"));
        m_plot.setDomainValueFormat(new DecimalFormat("#"));

   //     final PlotStatistics histStats = new PlotStatistics(1000, false);
   //     m_plot.addListener(histStats);

        // Software or Hardware acceleration
    //    m_plot.setLayerType(View.LAYER_TYPE_NONE, null);
    }

    LineAndPointFormatter makeFormatter(int width, int color1, int color2, float x2, float y2, boolean fill) {
        LineAndPointFormatter formatter = new LineAndPointFormatter(color1, null, color2, null);
        Paint linePaint = formatter.getLinePaint();
        linePaint.setStrokeWidth(width);
        linePaint.setShadowLayer(width, 1.0f, 1.0f, Color.argb(128, 0, 0, 0));

        if (fill) {
            Paint lineFill = new Paint();
            lineFill.setShader(new LinearGradient(0, 0, x2, y2, color2, color1, Shader.TileMode.MIRROR));
            lineFill.setAlpha(128+32);
            formatter.setFillPaint(lineFill);
            formatter.setLinePaint(linePaint);
        } else {
            formatter.setFillPaint(null);
            linePaint.setShader(new LinearGradient(0, 0, x2, y2, color2, color1, Shader.TileMode.MIRROR));
            formatter.setLinePaint(linePaint);
        }

        return formatter;
    }

    // ============================================================================================
    // SensorEventListener methods

    // Called whenever a new orSensor reading is taken.
    @Override
    public synchronized void onSensorChanged(SensorEvent sensorEvent) {
        m_sensorEvents.put(sensorEvent.sensor.getName(), sensorEvent);
    }

    private void add(SimpleXYSeries series, Number x, Number y) {
        if (series.size() > HISTORY_SIZE)
            series.removeFirst();

        series.addLast(x, y);
    }

    private void updatePlot() {

        if (m_plot == null)
            createGraph(m_rootView, null);
            // Add  sensor sample to series.
        Date now = new Date();
        final int NO_LIGHT = 0;
        final int MaxAgeMsec = 5000;

        for (String sensorName : m_sensorEvents.keySet()) {
            SensorEvent sensorEvent = m_sensorEvents.get(sensorName);
            if (sensorEvent != null) {
                long nowMsec =  SystemClock.uptimeMillis(); //  not System.currentTimeMillis() nor  SystemClock.elapsedRealtime();
                long eventMsec = sensorEvent.timestamp / 1000000;
                long deltaMsec = nowMsec - eventMsec;

                if (sensorName.contains(ORIENTATION_STR) && m_seriesOrientation_az != null) {
                    add(m_seriesOrientation_az, null, sensorEvent.values[0]);
                    add(m_seriesOrientation_pitch, null, sensorEvent.values[1]);
                    add(m_seriesOrientation_roll, null, sensorEvent.values[2]);
                    m_plotValues.put(ORIENTATION_STR, String.format("%.0f, %.0f, %.0f",
                            sensorEvent.values[0], sensorEvent.values[1], sensorEvent.values[2]));
                } else if (sensorName.contains(LIGHT_STR) && m_seriesLight != null) {
                    int lux = (deltaMsec < MaxAgeMsec) ? (int)sensorEvent.values[0] : NO_LIGHT;
                    lux = Math.min(lux, MAX_LIGHT);
                    add(m_seriesLight, null, lux);
                    if (deltaMsec > MaxAgeMsec) {
                        m_plotValues.put(LIGHT_STR, "Light N/A");
                    } else {
                        m_plotValues.put(LIGHT_STR, String.format("Lux %d", lux));
                    }
                }
            }
        }

        if (m_soundMeter != null) {
            double dbValue = m_soundMeter.getAmplitude();
            double avgDb = 0;
            double maxDb = 0;
            add(m_seriesAudio, null, dbValue);

            int lastIdx = m_seriesAudioMax.size();
            if (lastIdx == 0) {
                add(m_seriesAudioMax, null, dbValue);
                add(m_seriesAudioAvg, null, dbValue);
            } else {
                final int avgSpan = 20;
                int avgCnt = 0;
                for (int idx = Math.max(0, lastIdx - avgSpan); idx < lastIdx; idx++) {
                    avgDb += m_seriesAudio.getY(idx).doubleValue();
                    avgCnt++;
                }
                avgDb = avgDb / avgCnt;
                add(m_seriesAudioAvg, null, avgDb);

                final int maxSpan = 20;
                for (int idx = Math.max(0, lastIdx - maxSpan); idx < lastIdx; idx++) {
                    maxDb = Math.max(maxDb, m_seriesAudio.getY(idx).doubleValue());
                }
                add(m_seriesAudioMax, null, maxDb);

                if (m_sensorName.equals(AUDIO_STR)) {
                    // Adjust graph Range to match data.
                    double totAvgDb = 0;
                    maxDb = 0;
                    for (int idx = 0; idx < lastIdx; idx++) {
                        totAvgDb += m_seriesAudio.getY(idx).doubleValue();
                        maxDb = Math.max(maxDb, m_seriesAudio.getY(idx).doubleValue());
                    }
                    totAvgDb /= lastIdx;
                    maxDb = (maxDb * 9 + totAvgDb) / 10;
                    final long AUDIO_STEP = 1000;
                    if (maxDb > m_maxAudio) {
                        m_maxAudio = (long) ((maxDb + AUDIO_STEP - 1) / AUDIO_STEP) * AUDIO_STEP;
                        m_plot.setRangeBoundaries(0, m_maxAudio, BoundaryMode.FIXED);
                    } else if (maxDb <= m_maxAudio * 0.5) {
                        m_maxAudio = (long) (maxDb / AUDIO_STEP) * AUDIO_STEP;
                        m_plot.setRangeBoundaries(0, m_maxAudio, BoundaryMode.FIXED);
                    }
                }
            }
            m_plotValues.put(AUDIO_STR, String.format("%.0f Avg:%.0f", dbValue, avgDb));
        }

        WifiManager wifiMgr = (WifiManager) getActivity().getSystemService(Context.WIFI_SERVICE);
        if (wifiMgr != null && wifiMgr.isWifiEnabled() && wifiMgr.getDhcpInfo() != null && m_seriesWifi != null) {
            WifiInfo wifiInfo = wifiMgr.getConnectionInfo();
            int numberOfLevels = 10;
            int level = WifiManager.calculateSignalLevel(wifiInfo.getRssi(), numberOfLevels + 1);
            int wifiLevel = 100 * level / numberOfLevels;
            add(m_seriesWifi, null, wifiLevel);
            m_plotValues.put(WIFI_STR, String.valueOf(wifiLevel));
        }

        Intent batteryIntent = getActivity().getApplicationContext().registerReceiver(null, new IntentFilter(Intent.ACTION_BATTERY_CHANGED));
        if (batteryIntent != null && m_seriesBattery != null) {
            int level = batteryIntent.getIntExtra(BatteryManager.EXTRA_LEVEL, 0);
            int scale = batteryIntent.getIntExtra(BatteryManager.EXTRA_SCALE, 100);
            int batteryLevel = level * 100 / scale;
            add(m_seriesBattery, null, batteryLevel);
            m_plotValues.put(BATTERY_STR, String.valueOf(batteryLevel));
        }

        ActivityManager actMgr = (ActivityManager) getActivity().getSystemService(Context.ACTIVITY_SERVICE);
        if (actMgr != null && m_seriesProcCnt != null) {
            try {
                final int MAX_PROC_CNT = 200;
                int processCnt = actMgr.getRunningAppProcesses().size();
                add(m_seriesProcCnt, null, Math.min(processCnt, MAX_PROC_CNT));
                m_plotValues.put(PROCESSES_STR, String.valueOf(processCnt));
            } catch (Exception ex) {
            }
        }

        if (Build.VERSION.SDK_INT >= 16 && m_seriesFreeMem != null) {
            //   Total Memory API >= 16
            try {
                ActivityManager.MemoryInfo mi = new ActivityManager.MemoryInfo();
                ActivityManager activityManager = (ActivityManager)
                        getActivity().getSystemService(Context.ACTIVITY_SERVICE);
                activityManager.getMemoryInfo(mi);
                // long heapUsing = Debug.getNativeHeapSize();
                long freeMem = 100 * mi.availMem / mi.totalMem;
                add(m_seriesFreeMem, null, freeMem);
                m_plotValues.put(MEMORY_STR, String.valueOf(freeMem));
            } catch (Exception ex) {
            }
        }

        if (m_plotValues.get(m_sensorName) != null)
            m_valueTv1.setText(m_plotValues.get(m_sensorName));
    }


    @Override
    public void onAccuracyChanged(Sensor sensor, int i) {
        // Not interested in this event
    }

	// ============================================================================================
	// Internal methods
}